
# q-expansion of newform 612.4.a.b, downloaded from the LMFDB on 02 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 612
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [3], [-16], [57], [-25], [-17], [-13], [93], [6], [110], [248], [333], [-115], [294], [318], [30], [668], [-220], [-540], [1214], [-442], [438], [-60], [1568], [-552], [1001], [237], [1388], [1023], [-97], [2103], [3018], [-2518], [-882], [-1264], [11], [-298], [2121], [-3201], [246], [-4090], [2106], [-274], [483], [-5104], [1886], [-1213], [207], [-466], [3717], [-3780], [2348], [-2868], [2664], [624], [-7431], [-8449], [-3130], [984], [-106], [-180], [-1636], [-5880], [608], [6618], [-5017], [-4114], [-12012], [9053], [2694], [3516], [-76], [-7390], [-3100], [-7068], [-504], [6212], [14865], [-11875], [9036], [10967], [-7824], [-2329], [-14236], [17370], [-10422], [16949], [2718], [8840], [9786], [9489], [7202], [-378], [16814], [-20373], [9492], [16677], [-4708], [11996], [15980], [-15654], [-24042], [8040], [-256], [-14359], [3060], [-22278], [20922], [-4390], [-19258], [26267], [10530], [19574], [-20329], [23271], [20648], [-678], [-24453], [20118], [22649], [18158], [-4731], [-23145], [-2644], [2808], [-12046], [7743], [-15712], [8354], [-12949], [-15072], [-25978], [-24379], [10458], [7289], [-30336], [3920], [792], [-11571], [-17710], [24243], [19928], [42279], [10910], [-5061], [-23758], [-4170], [-14452], [-46284], [21230], [-51246], [26783], [-20853], [17786], [45081], [21791], [43503], [-38758], [-1446], [-29100], [31452], [-12835], [-30894], [-51534], [2499], [48284], [-40570], [-9796], [17670], [-50310], [-41011], [-61371], [30251], [3191], [-23757], [17516], [62673], [28169], [-17161], [-22735], [-23316], [-23650], [69000], [44070], [19002], [-3058], [69281], [-53098], [17340], [-57262], [2508], [70067], [45564], [-55509], [-40650], [68738], [-38260], [47010], [-57378], [-31503], [-69580], [-52591], [34673], [-10920], [62850], [-59128], [27636], [-61251], [23708], [-34378], [38442], [71738], [74556], [91947], [-27412], [42761], [19188], [-62481], [18240], [52385], [-71296], [7356], [48638], [-92850], [11702], [14931], [-8811], [16805], [336], [107519], [-92302], [4412], [-105822], [60584], [-26574], [32876], [-102951], [48645], [1896], [25569], [48692], [28907], [-98677], [5739], [-75147], [-100894], [-9873], [-655], [-1752], [11369], [75450], [34776], [39956], [40650], [-99936], [91892], [87896], [128019], [-50146], [-97210], [8364], [-59692], [-73192], [-98943], [-70441], [105462], [-40920], [-126628], [-103830], [-78472], [-100987], [30359], [-107608], [-133306], [-130669], [22176], [30413], [132221], [-61140], [132444], [99290], [-86904], [4262], [-50446], [94296], [46970], [1554], [-74425], [-36744], [-77685], [-106647], [40998], [134511], [-68416], [32655], [-117352], [-26328], [72237], [51692], [148769], [119736], [-137320], [130053], [9788], [50006], [-37014], [23672], [-57936], [-3598], [-64788], [-52986], [-105837], [66023], [-7716], [-51793], [-35262], [-64137], [23456], [75600], [34970], [52298], [41454], [118127], [162126], [-96661], [-143746], [-58030], [-156672], [110517], [86636], [-84663], [-68734], [-15114], [52580], [24969], [149873], [30903], [-11266], [-40111], [-56899], [996], [-27861], [88307], [145020], [120801], [-141370], [1379], [182037], [187374], [198809], [-6388], [125604], [180938], [73703], [67404], [-133464], [-9660], [120339], [60534], [85400], [-13935], [41820], [-126036], [-17551], [-207343], [-72531], [-136252], [-104512], [-20232], [-70924], [97059], [-76980], [-67240], [-134536], [101775], [-240708], [178535], [-91926], [-182959], [-210513], [-144600], [-19204], [-246393], [193694], [145545], [-54475], [50438], [-143164], [-155940], [9470], [-45951], [246984], [-209341], [80046], [232700], [-61888], [-102270], [153596], [-192546], [-93004], [213900], [-137059], [157974], [-20310], [180902], [250214], [-98988], [-116335], [-183915], [-51568], [-228414], [123600], [158564], [-103831], [-186726], [-281715], [140090], [-22377], [11058], [217098], [-23650], [-144561], [-92010], [-10852], [121176], [-40752], [-92139], [-65347], [-30189]]
